@@ -1,8 +1,12 @@
 
 from collections import namedtuple
 
-from ..util.iter import namespace
+from typing import Dict
+from typing import List
+
+from ..util.iter import namespace_cls
 from ..util.iter import is_header_row
+
 from ..util.text import format_header
 from ..util.text import format_header_lite
 from ..util.text import format_integer
@@ -31,6 +35,9 @@ class flux_row_cls:
         properties must be set on self.__dict__ instead of directly on self to prevent
         premature __setattr__ lookups
         """
+        self._headers: Dict[str, int]
+        self.values:   List
+
         self.__dict__['_headers'] = headers
         self.__dict__['values']   = values
 
@@ -41,9 +48,6 @@ class flux_row_cls:
         PyCharm will recognize the ndarray and enable the "...view as array"
         option in the debugger which displays values in a special window as a table
         """
-        if not numpy_installed:
-            raise ImportError('numpy site-package not installed')
-
         names  = [format_header(n) for n in self.header_names()]
         values = list(self.__dict__['values'])
 
@@ -58,9 +62,18 @@ class flux_row_cls:
             names.insert(0,  '⟨address⟩')
             values.insert(0, '⟨{:,}⟩'.format(self.__dict__['r_i']).replace(',', '_'))
 
-        # noinspection PyUnresolvedReferences
-        return numpy.transpose([numpy.array(names,  dtype=object),
-                                numpy.array(values, dtype=object)])
+        m = list(zip(names, values))
+
+        if not numpy_installed:
+            _nf_ = max([len(n) for n in names])  + 1
+            _nf_ = '{: <%s}' % str(_nf_)
+
+            m = [' '.join([_nf_.format(n), v]) for n, v in m]
+            m = '\n'.join(m)
+
+            return m
+
+        return numpy.array(m, dtype=object)
 
     @property
     def headers(self):
@@ -85,9 +98,12 @@ class flux_row_cls:
         return is_header_row(self.__dict__['values'],
                              self.__dict__['_headers'])
 
+    def dict(self):
+        return ordereddict(zip(self.header_names(), self.__dict__['values']))
+
     def namedrow(self):
         d = ordereddict(zip(self.header_names(), self.__dict__['values']))
-        return namespace(**d)
+        return namespace_cls(**d)
 
     # noinspection PyArgumentList
     def namedtuple(self):
@@ -202,27 +218,23 @@ class flux_row_cls:
     def __hash__(self):
         return id(self.__dict__['_headers']) + hash(tuple(self.__dict__['values']))
 
-    def __getstate__(self):
-        return self.__dict__
-
-    def __setstate__(self, __dict__):
-        self.__dict__.update(__dict__)
-
     def __repr__(self):
         if 'address' in self.__dict__:
             row_label = format_header_lite(self.__dict__['address'])
-            row_label = '{}   '.format(row_label)
+            row_label = ' {}   '.format(row_label)
         elif 'r_i' in self.__dict__:
             row_label = format_integer(self.__dict__['r_i'])
             row_label = format_header_lite(row_label)
-            row_label = '{}   '.format(row_label)
+            row_label = ' {}   '.format(row_label)
         else:
             row_label = ''
 
         if self.is_jagged():
-            is_jagged = '🗲jagged🗲  '
+            jagged_label = len(self.__dict__['values']) - len(self.__dict__['_headers'])
+            jagged_label = format_header_lite('{:+}'.format(jagged_label))
+            jagged_label = ' 🗲jagged {}🗲  '.format(jagged_label)
         else:
-            is_jagged = ''
+            jagged_label = ''
 
         if self.is_header_row():
             values = ', '.join(str(n) for n in self.header_names())
@@ -231,7 +243,7 @@ class flux_row_cls:
             values = (repr(self.__dict__['values']).replace('"', '')
                                                    .replace("'", ''))
 
-        return ' {}{}{}'.format(row_label, is_jagged, values)
+        return '{}{}{}'.format(row_label, jagged_label, values)
 
     @staticmethod
     def __raise_attribute_error(invalid, headers):
@@ -243,6 +255,3 @@ class flux_row_cls:
 
         raise AttributeError(s) from None
 
-
-
-
