@@ -1,9 +1,13 @@
 
 import re
-# noinspection PyUnresolvedReferences
 
+# noinspection PyUnresolvedReferences
 from pythoncom import com_error as pythoncom_error
 
+from typing import Generator
+from typing import List
+from typing import Any
+
 from .lev_row_cls import lev_row_cls
 
 from .. import excel_address
@@ -11,7 +15,7 @@ from .. import worksheet
 from .. excel_constants import *
 
 from ... util.iter import iterator_to_collection
-from ... util.iter import map_values_to_enum
+from ... util.iter import inverted_enumerate
 from ... util.iter import modify_iteration_depth
 from ... util.text import object_name
 
@@ -171,33 +175,33 @@ class excel_levity_cls:
 
         return worksheet.is_range_empty(self.ws.Range(a))
 
-    def rows(self, r_1='*h', r_2='*l'):
+    def rows(self, r_1='*h', r_2='*l') -> Generator[List, Any, Any]:
         if self.is_empty():
-            return [[]]
+            return ([] for _ in range(1))
 
         a = '*f {}:*l {}'.format(r_1, r_2)
-        excel_range = self.excel_range(a)
+        excel_range = self.range(a)
 
-        return worksheet.escape_excel_range_errors(excel_range)
-
-    def lev_rows(self, r_1='*h', r_2='*l'):
-        if self.is_empty():
-            return [[]]
+        return (row for row in worksheet.escape_excel_range_errors(excel_range))
 
+    def lev_rows(self, r_1='*h', r_2='*l') -> Generator[lev_row_cls, Any, Any]:
         if self.headers:
-            headers = map_values_to_enum(self.headers.keys())
+            headers = inverted_enumerate(self.headers.keys())
         elif self.m_headers:
-            headers = map_values_to_enum(self.m_headers.keys())
+            headers = inverted_enumerate(self.m_headers.keys())
         else:
             headers = ordereddict()
 
+        if self.is_empty():
+            return (lev_row_cls(headers, [], '') for _ in range(1))
+
         reserved = headers.keys() & set(lev_row_cls.reserved_names())
         if reserved:
             raise NameError("reserved name(s) {} found in header row {}"
                             .format(list(reserved), list(headers.keys())))
 
         a = '*f {}:*l {}'.format(r_1, r_2)
-        excel_range = self.excel_range(a)
+        excel_range = self.range(a)
 
         r_1 = excel_range.Row
         c_1, c_2 = self.first_c, self.last_c
@@ -230,23 +234,23 @@ class excel_levity_cls:
             r = '*f'
 
         a = '{} {}:{} {}'.format(c_1, r, c_2, r)
-        excel_range = self.excel_range(a)
+        excel_range = self.range(a)
 
         if excel_range.Cells.Count == 1:
             a = '{} {}:{} {}'.format(c_1, r, c_2, self.last_r + 1)
-            excel_range = self.excel_range(a)
+            excel_range = self.range(a)
 
         self.remove_filter()
         excel_range.AutoFilter(*(1,))
 
     def calculate(self):
-        self.excel_range('*f *h: *l *l').Calculate()
+        self.range('*f *h: *l *l').Calculate()
 
     def clear(self, reference,
                     clear_values=True,
                     clear_colors=False):
 
-        excel_range = self.excel_range(reference)
+        excel_range = self.range(reference)
         _, r_1, _, r_2 = worksheet.parse_range(excel_range)
 
         if clear_values:
@@ -328,14 +332,14 @@ class excel_levity_cls:
             return ordereddict()
 
         c_1 = excel_range.Column
-        headers = map_values_to_enum(row, c_1)
+        headers = inverted_enumerate(row, c_1)
         headers = ordereddict((h, excel_address.col_letter(v)) for h, v in headers.items())
 
         return headers
 
     def __index_headers(self, row_ref):
         a = '*f {} :*l {}'.format(row_ref, row_ref)
-        excel_range = self.excel_range(a)
+        excel_range = self.range(a)
 
         return self.__index_row_headers(excel_range)
 
@@ -351,7 +355,7 @@ class excel_levity_cls:
 
         self.headers = self.__index_headers('header_r')
 
-    def excel_range(self, reference):
+    def range(self, reference):
         if not self.is_worksheet_type:
             ws_type = object_name(self.ws)
             raise TypeError('{} is not an Excel worksheet '.format(ws_type))
@@ -381,11 +385,11 @@ class excel_levity_cls:
         return a
 
     def __getitem__(self, reference):
-        return self.excel_range(reference)
+        return self.range(reference)
 
     def __setitem__(self, reference, v):
         """ write value(s) to excel range """
-        excel_range  = self.excel_range(reference)
+        excel_range  = self.range(reference)
 
         m = self.__validate_matrix_within_range_boundaries(v, excel_range)
 
@@ -399,17 +403,20 @@ class excel_levity_cls:
         if was_filtered:
             self.reapply_filter()
 
-    def __iter__(self):
-        for row in self.lev_rows('*f'):
-            yield row
+    def __iter__(self) -> Generator[lev_row_cls, Any, Any]:
+        return self.lev_rows('*f')
 
     def __repr__(self):
         if not self.is_worksheet_type:
             return "{{}}: '{}'".format(self.ws.__class__.__name__, self.ws_name)
 
         if self.first_c and self.last_c:
+            r = max(self.header_r, self.header_r)
+            if r == 0:
+                r = self.first_r
+
             a = "{}{}:{}{}".format(self.first_c,
-                                   self.header_r,
+                                   r,
                                    self.last_c,
                                    self.last_r)
         else:
